package ast

import (
	"git.akyoto.dev/cli/q/src/build/errors"
	"git.akyoto.dev/cli/q/src/build/expression"
	"git.akyoto.dev/cli/q/src/build/token"
)

// parseKeyword generates a keyword node from an instruction.
func parseKeyword(tokens token.List, source []byte, nodes AST) (Node, error) {
	switch tokens[0].Kind {
	case token.Assert:
		if len(tokens) == 1 {
			return nil, errors.New(errors.MissingExpression, nil, tokens[0].End())
		}

		condition := expression.Parse(tokens[1:])
		return &Assert{Condition: condition}, nil

	case token.If:
		blockStart, _, body, err := block(tokens, source)
		condition := expression.Parse(tokens[1:blockStart])
		return &If{Condition: condition, Body: body}, err

	case token.Else:
		_, _, body, err := block(tokens, source)

		if len(nodes) == 0 {
			return nil, errors.New(errors.ExpectedIfBeforeElse, nil, tokens[0].Position)
		}

		last := nodes[len(nodes)-1]
		ifNode, exists := last.(*If)

		if !exists {
			return nil, errors.New(errors.ExpectedIfBeforeElse, nil, tokens[0].Position)
		}

		ifNode.Else = body
		return nil, err

	case token.Loop:
		_, _, body, err := block(tokens, source)
		return &Loop{Body: body}, err

	case token.Return:
		if len(tokens) == 1 {
			return &Return{}, nil
		}

		value := expression.Parse(tokens[1:])
		return &Return{Value: value}, nil

	default:
		return nil, errors.New(&errors.KeywordNotImplemented{Keyword: tokens[0].Text(source)}, nil, tokens[0].Position)
	}
}

// block retrieves the start and end position of a block.
func block(tokens token.List, source []byte) (blockStart int, blockEnd int, body AST, err error) {
	blockStart = tokens.IndexKind(token.BlockStart)
	blockEnd = tokens.LastIndexKind(token.BlockEnd)

	if blockStart == -1 {
		err = errors.New(errors.MissingBlockStart, nil, tokens[0].End())
		return
	}

	if blockEnd == -1 {
		err = errors.New(errors.MissingBlockEnd, nil, tokens[len(tokens)-1].End())
		return
	}

	body, err = Parse(tokens[blockStart+1:blockEnd], source)
	return
}
