package core

import (
	"fmt"

	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/ast"
	"git.akyoto.dev/cli/q/src/build/config"
)

// CompileAssert compiles an assertion.
func (f *Function) CompileAssert(assert *ast.Assert) error {
	if config.SkipAsserts {
		return nil
	}

	f.count.assert++
	success := fmt.Sprintf("%s_assert_%d_true", f.Name, f.count.assert)
	fail := fmt.Sprintf("%s_assert_%d_false", f.Name, f.count.assert)
	err := f.CompileCondition(assert.Condition, success, fail)

	if err != nil {
		return err
	}

	f.AddLabel(success)

	f.Defer(func() {
		f.AddLabel(fail)
		f.Jump(asm.JUMP, "_crash")
	})

	return err
}
