package core

import (
	"git.akyoto.dev/cli/q/src/build/ast"
	"git.akyoto.dev/cli/q/src/build/errors"
	"git.akyoto.dev/cli/q/src/build/token"
)

// CompileAssign compiles an assign statement.
func (f *Function) CompileAssign(node *ast.Assign) error {
	operator := node.Expression.Token
	left := node.Expression.Children[0]
	right := node.Expression.Children[1]

	if left.IsLeaf() {
		name := left.Token.Text(f.File.Bytes)
		variable := f.VariableByName(name)

		if variable == nil {
			return errors.New(&errors.UnknownIdentifier{Name: name}, f.File, left.Token.Position)
		}

		defer f.UseVariable(variable)
		return f.Execute(operator, variable.Register, right)
	}

	if left.Token.Kind == token.Array {
		return f.CompileAssignArray(node)
	}

	if left.Token.Kind == token.Separator && right.Token.Kind == token.Div {
		return f.CompileAssignDivision(node)
	}

	return errors.New(errors.NotImplemented, f.File, left.Token.Position)
}
