package core

import (
	"fmt"

	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/ast"
)

// CompileIf compiles a branch instruction.
func (f *Function) CompileIf(branch *ast.If) error {
	f.count.branch++

	var (
		end     string
		success = fmt.Sprintf("%s_if_%d_true", f.Name, f.count.branch)
		fail    = fmt.Sprintf("%s_if_%d_false", f.Name, f.count.branch)
		err     = f.CompileCondition(branch.Condition, success, fail)
	)

	if err != nil {
		return err
	}

	f.AddLabel(success)
	f.PushScope(branch.Body, f.File.Bytes)
	err = f.CompileAST(branch.Body)

	if err != nil {
		return err
	}

	if branch.Else != nil {
		end = fmt.Sprintf("%s_if_%d_end", f.Name, f.count.branch)
		f.Jump(asm.JUMP, end)
	}

	f.PopScope()
	f.AddLabel(fail)

	if branch.Else != nil {
		f.PushScope(branch.Else, f.File.Bytes)
		err = f.CompileAST(branch.Else)

		if err != nil {
			return err
		}

		f.PopScope()
		f.AddLabel(end)
		return nil
	}

	return nil
}
