package core

import (
	"git.akyoto.dev/cli/q/src/build/arch/x64"
	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/cpu"
	"git.akyoto.dev/cli/q/src/build/fs"
	"git.akyoto.dev/cli/q/src/build/register"
	"git.akyoto.dev/cli/q/src/build/scope"
	"git.akyoto.dev/cli/q/src/build/token"
)

// NewFunction creates a new function.
func NewFunction(pkg string, name string, file *fs.File, body []token.Token) *Function {
	return &Function{
		Package: pkg,
		Name:    name,
		File:    file,
		Body:    body,
		Machine: register.Machine{
			Assembler: asm.Assembler{
				Instructions: make([]asm.Instruction, 0, 8),
			},
			Stack: scope.Stack{
				Scopes: []*scope.Scope{{}},
			},
			CPU: cpu.CPU{
				All:     x64.AllRegisters,
				Input:   x64.CallRegisters,
				General: x64.GeneralRegisters,
				Syscall: x64.SyscallRegisters,
				Output:  x64.ReturnValueRegisters,
			},
		},
	}
}
