package cpu_test

import (
	"testing"

	"git.akyoto.dev/cli/q/src/build/cpu"
	"git.akyoto.dev/go/assert"
)

func TestRegisterState(t *testing.T) {
	s := cpu.State{}
	assert.False(t, s.IsReserved(0))
	assert.False(t, s.IsUsed(0))
	s.Reserve(0)
	assert.True(t, s.IsReserved(0))
	assert.False(t, s.IsUsed(0))
	s.Use(0)
	assert.True(t, s.IsReserved(0))
	assert.True(t, s.IsUsed(0))
	s.Free(0)
	assert.False(t, s.IsReserved(0))
	assert.False(t, s.IsUsed(0))
}

func TestFindFree(t *testing.T) {
	s := cpu.State{}
	s.Reserve(0)
	s.Use(1)

	reg, found := s.FindFree([]cpu.Register{0, 1, 2, 3})
	assert.True(t, found)
	assert.Equal(t, reg, 2)

	_, found = s.FindFree([]cpu.Register{0, 1})
	assert.False(t, found)
}

func TestMustFindFree(t *testing.T) {
	s := cpu.State{}
	s.Reserve(0)
	s.Use(1)

	reg := s.MustFindFree([]cpu.Register{0, 1, 2, 3})
	assert.Equal(t, reg, 2)
}
